'''
Created on Oct 16, 2017

@author: khiem
'''
# example_consumer.py
import pika, os, time

def pdf_process_function(msg):
    print(" PDF processing")
    print(" Received %r" % msg)

    time.sleep(5)  # delays for 5 seconds
    print(" PDF processing finished")

# Access the CLODUAMQP_URL environment variable and parse it (fallback to localhost)
url = os.environ.get(
    'CLOUDAMQP_URL',
    'queue_message://cfpzarme:MqsdI8OBcuTy_C1C1JP1B2a8APAWQzf1@mustang.rmq.cloudamqp.com/cfpzarme'
    )

params = pika.URLParameters(url)
connection = pika.BlockingConnection(params)
channel = connection.channel()  # start a channel
channel.queue_declare(queue='pdfprocess')  # Declare a queue

# create a function which is called on incoming messages
def callback(ch, method, properties, body):
    pdf_process_function(body)

# set up subscription on the queue
# channel.basic_consume(callback, queue='pdfprocess', no_ack=True)
print channel.basic_get(queue='ina_casp_service_listen1', no_ack=False)

# start consuming (blocks)
channel.start_consuming()
connection.close()



'''
Created on Nov 26, 2017

@author: khiem
'''
from pyspark import SparkConf, SparkContext
from pyspark.ml.feature import StringIndexer
from pyspark import SQLContext

# Configure the Spark environment
sparkConf = SparkConf().setAppName("Classification").setMaster("local")
sc = SparkContext(conf=sparkConf)
sqlC = SQLContext(sc)

# StringIndexer
df = sqlC.createDataFrame(
    [(0, "a"), (1, "b"), (2, "c"), (3, "a"), (4, "a"), (5, "c")],
    ["id", "category"])
indexer = StringIndexer(inputCol="category", outputCol="indexer")
print indexer
print indexer.fit(df)
indexed = indexer.fit(df).transform(df)
indexed.show()




'''
Created on Dec 5, 2017

@author: khiem
'''
import paho.mqtt.client as mqtt
import os, urlparse

# Define event callbacks
def on_connect(client, userdata, flags, rc):
    print("rc: " + str(rc))

    # Subscribing in on_connect() means that if we lose the connection and
    # reconnect then subscriptions will be renewed.
    client.subscribe("$SYS/#")

def on_message(client, obj, msg):
    print '[x] get msg'
    print(msg.topic + " " + str(msg.qos) + " " + str(msg.payload))

def on_publish(client, obj, mid):
    print '[x] post msg'
    print("mid: " + str(mid))

def on_subscribe(client, obj, mid, granted_qos):
    print("Subscribed: " + str(mid) + " " + str(granted_qos))

def on_log(client, obj, level, string):
    print(string)

mqttc = mqtt.Client()

# Assign event callbacks
mqttc.on_message = on_message
mqttc.on_connect = on_connect
mqttc.on_publish = on_publish
mqttc.on_subscribe = on_subscribe

# Uncomment to enable debug messages
mqttc.on_log = on_log

# Parse CLOUDMQTT_URL (or fallback to localhost)
url_str = 'mqtt://ixxqfseh:My4LGakulUjR@m14.cloudmqtt.com:16639'
url = urlparse.urlparse(url_str)
topic = 'test'

# Connect
mqttc.username_pw_set(url.username, url.password)
mqttc.connect(url.hostname, url.port)

# Start subscribe, with QoS level 0
mqttc.subscribe(topic, 0)

# Publish a message
mqttc.publish(topic, "my message: hello all")

# Continue the network loop, exit when an error occurs
rc = 0
while rc == 0:
    rc = mqttc.loop()
print("rc: " + str(rc))


'''
Created on Jan 23, 2018

@author: khiem
'''
from pyspark import SparkConf, SparkContext
from pyspark.mllib.linalg import Vectors
from pyspark.mllib.linalg.distributed import RowMatrix

# Configure the Spark environment
sparkConf = SparkConf().setAppName("PCA").setMaster("local")
sc = SparkContext(conf=sparkConf)


# row matrix
# rows = sc.parallelize([[1, 2], [1, 5]])
# mat = RowMatrix(rows)
# sims = mat.columnSimilarities()
# print sims.entries.first().value

# PCA
rows = sc.parallelize([
    Vectors.sparse(5, {1: 1.0, 3: 7.0}),
    Vectors.dense(2.0, 0.0, 3.0, 4.0, 5.0),
    Vectors.dense(4.0, 0.0, 0.0, 6.0, 7.0)
])

mat = RowMatrix(rows)
# Compute the top 4 principal components.
# Principal components are stored in a local dense matrix.
pc = mat.computePrincipalComponents(2)
print pc  # orth matrix

# Project the rows to the linear space spanned by the top 4 principal components.
projected = mat.multiply(pc)
print projected.rows.collect()





# kmean real example

from pyspark import SparkConf, SparkContext
from pyspark.sql import SQLContext
from pyspark.ml.feature import OneHotEncoder, StringIndexer, VectorAssembler
from pyspark.ml.clustering import KMeans
from pyspark.ml import Pipeline

# Configure the Spark environment
sparkConf = SparkConf().setAppName("Classification").setMaster("local")
sc = SparkContext(conf=sparkConf)

sqlContext = SQLContext(sc)

input = sqlContext.createDataFrame([
    ("a@email.com", 12000, "M"),
    ("b@email.com", 43000, "M"),
    ("c@email.com", 5000, "F"),
    ("d@email.com", 60000, "M")
]).toDF("email", "income", "gender")

indexer = StringIndexer().setInputCol("gender").setOutputCol("genderIndex")
encoder = OneHotEncoder().setInputCol("genderIndex").setOutputCol("genderVec")
assembler = VectorAssembler().setInputCols(["income", "genderVec"]).setOutputCol("features")
kmeans = KMeans().setK(2).setFeaturesCol("features").setPredictionCol("prediction") 
pipeline = Pipeline().setStages([indexer, encoder, assembler, kmeans])

kMeansPredictionModel = pipeline.fit(input)
print kMeansPredictionModel

predictionResult = kMeansPredictionModel.transform(input)
predictionResult.show()



'''
Created on Jan 6, 2018

@author: khiem

config
<source>
  @type forward
  port 24224
</source>
<match fluentd.test.**>
  @type stdout
</match>

out
2018-01-07T21:24:24+07:00 ina.casp.start: {"to":"userB","from":"userA"}

'''
# from fluent import sender
# from fluent import event
# sender.setup('ina', host='localhost', port=24224)
# event.Event('start', {
#     'time': '2018-01-08 11:25:47',
#     'level': 'INFO',
#     'tag': 'CASP',
#     'message': 'message hello2'
# })


import logging
from fluent import handler

custom_format = {
  'host': '%(hostname)s',
  'where': '%(module)s.%(funcName)s',
  'level': '%(levelname)s',
  'stack_trace': '%(exc_text)s'
}

logging.basicConfig(level=logging.INFO)
l = logging.getLogger('fluent.testing')
h = handler.FluentHandler(
    'ina.casp', host='localhost', port=24224,
    buffer_overflow_handler='overflow_handler'
)
formatter = handler.FluentRecordFormatter(custom_format)
h.setFormatter(formatter)
l.addHandler(h)
l.info("This log entry will be logged with the additional key: message.")



'''
Created on Oct 16, 2017

@author: khiem
'''
# example_publisher.py
import pika, os, logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


# Parse CLODUAMQP_URL (fallback to localhost)
url = os.environ.get(
    'CLOUDAMQP_URL',
    'queue_message://caujhpxa:gOLp-uuHnVfzoXQqKtFPdx2B8yWj46HR@mustang.rmq.cloudamqp.com/caujhpxa'
    )
params = pika.URLParameters(url)
params.socket_timeout = 5

connection = pika.BlockingConnection(params)  # Connect to CloudAMQP
channel = connection.channel()  # start a channel
channel.queue_declare(queue='pdfprocess')  # Declare a queue
logger.info('ddd')
# send a message
channel.basic_publish(exchange='', routing_key='pdfprocess', body='User information')
print ("[x] Message sent to consumer")

if connection.is_open:
    channel.close()
# connection.close()



'''
Created on Dec 6, 2017

@author: khiem
'''
import os
os.environ['GOOGLE_APPLICATION_CREDENTIALS']='/home/khiem/workspace/INABachPhuBTSAnalytics-fa0fa8698768.json'

# Imports the Google Cloud client library
from google.cloud import bigquery

# Instantiates a client
bigquery_client = bigquery.Client()

# The name for the new dataset
dataset_id = 'my_new_dataset'

# Prepares a reference to the new dataset
dataset_ref = bigquery_client.dataset(dataset_id)
dataset = bigquery.Dataset(dataset_ref)

# Creates the new dataset
# dataset = bigquery_client.create_dataset(dataset)
# bigquery_client.delete_dataset(dataset)

# print('Dataset {} created.'.format(dataset.dataset_id))

# Query
query_job = bigquery_client.query("""
    #standardSQL
    SELECT corpus AS title, COUNT(*) AS unique_words
    FROM `bigquery-public-data.samples.shakespeare`
    GROUP BY title
    ORDER BY unique_words DESC
    LIMIT 10""")

results = query_job.result()  # Waits for job to complete.
print results
for row in results:
    print row
    print("{}: {}".format(row.title, row.unique_words))
    
    
    
    


'''
Created on Apr 26, 2017

@author: atc
https://fullstackml.com/beginners-guide-apache-spark-machine-learning-scenario-with-a-large-input-dataset-1f0c6bc53f8f
https://fullstackml.com/beginners-guide-apache-spark-python-machine-learning-scenario-with-a-large-input-dataset-3fd1c319bbc

'''
import re
import pyspark
from pyspark.sql.types import *
from pyspark.sql.functions import *
from pyspark import SparkConf, SQLContext, SparkContext
 
from pyspark.ml import Pipeline
from pyspark.ml.classification import LogisticRegression
from pyspark.ml.feature import HashingTF, Tokenizer
from pyspark.mllib.evaluation import BinaryClassificationMetrics
 
targetTag = "java"
sparkConf = SparkConf().setAppName("StackExchange").setMaster("local")
sc = SparkContext(conf = sparkConf)
textFile = sc.textFile("/home/atc/data_src/Posts.small.xml")
sqlcontext = SQLContext(sc)
 
postsXml = textFile.map( lambda line: line.strip() ).\
    filter( lambda line: line != "<posts>" and line != "</posts>").\
    filter( lambda line: not line.startswith("<?xml version=") ).\
    filter( lambda line: line.find("Id=") >= 0 ).\
    filter( lambda line: line.find("Tags=") >= 0 ).\
    filter( lambda line: line.find("Body=") >= 0 ).\
    filter( lambda line: line.find("Title=") >= 0 )
 
postsRDD = postsXml.map( lambda s: pyspark.sql.Row(\
                Id = re.search('Id=".+?"', s).group(0)[4:-1].encode('utf-8'),\
                Label = 1.0 if re.search('Tags=".+?"', s) != None\
                            and re.search('Tags=".+?"', s).group(0)[6:-1].encode('utf-8').find(targetTag) >= 0 else 0.0,\
                Text = ((re.search('Title=".+?"', s).group(0)[7:-1] if re.search('Title=".+?"', s) != None else "") + \
                        " " + (re.search('Body=".+?"', s).group(0)[6:-1]) if re.search('Body=".+?"', s) != None else "")))
 
postsLabeled = sqlcontext.createDataFrame(postsRDD)
 
positive = postsLabeled.filter(postsLabeled.Label > 0.0)
negative = postsLabeled.filter(postsLabeled.Label < 1.0)
 
# Sample without replacement (False)
positiveTrain = positive.sample(False, 0.9)
negativeTrain = negative.sample(False, 0.9)
training = positiveTrain.unionAll(negativeTrain)
 
negTrainTmp1 = negativeTrain.withColumnRenamed("Label", "Flag")
negativeTrainTmp = negTrainTmp1.select(negTrainTmp1.Id, negTrainTmp1.Flag)
 
 
negativeTest = negative.join( negativeTrainTmp, negative.Id == negativeTrainTmp.Id, "LeftOuter").\
                        filter("Flag is null").\
                        select(negative.Id, negative.Text, negative.Label)
 
posTrainTmp1 = positiveTrain.withColumnRenamed("Label", "Flag")
positiveTrainTmp = posTrainTmp1.select(posTrainTmp1.Id, posTrainTmp1.Flag)
 
positiveTest = positive.join( positiveTrainTmp, positive.Id == positiveTrainTmp.Id, "LeftOuter").\
                        filter("Flag is null").\
                        select(positive.Id, positive.Text, positive.Label)
testing = negativeTest.unionAll(positiveTest)
 
 
# CREATE MODEL
numFeatures = 20000
numEpochs = 20
regParam = 0.02
 
tokenizer = Tokenizer().setInputCol("Text").setOutputCol("Words")
hashingTF = HashingTF().setNumFeatures(numFeatures).\
                setInputCol(tokenizer.getOutputCol()).setOutputCol("Features")
lr = LogisticRegression().setMaxIter(numEpochs).setRegParam(regParam).\
                                    setFeaturesCol("Features").setLabelCol("Label").\
                                    setRawPredictionCol("Score").setPredictionCol("Prediction")
pipeline = Pipeline().setStages([tokenizer, hashingTF, lr])
 
# this comand takes a time
model = pipeline.fit(training)
 
testTitle = "Easiest way to merge a release into one JAR file"
testBody = """Is there a tool or script which easily merges a bunch of 
    href=&quot;http://en.wikipedia.org/wiki/JAR_%28file_format%29&quot;
    &gt;JAR&lt;/a&gt; files into one JAR file? A bonus would be to easily set the main-file manifest 
    and make it executable. I would like to run it with something like:
    &lt;/p&gt;&#xA;&#xA;&lt;blockquote&gt;&#xA;  &lt;p&gt;java -jar 
    rst.jar&lt;/p&gt;&#xA;&lt;/blockquote&gt;&#xA;&#xA;&lt;p&gt;
    As far as I can tell, it has no dependencies which indicates that it shouldn't be an easy 
    single-file tool, but the downloaded ZIP file contains a lot of libraries."""
testText = testTitle + testBody
testDF = sqlcontext.createDataFrame([ ("0", testText, 1.0)], ["Id", "Text", "Label"])
result = model.transform(testDF)
prediction = result.collect()[0][7]
print("Prediction: ", prediction)
 
 
testingResult = model.transform(testing)
testingResultScores = testingResult.select("Prediction", "Label").rdd.map( lambda r: (float(r[0]), float(r[1])))
bc = BinaryClassificationMetrics(testingResultScores)
roc = bc.areaUnderROC
print("Area under the ROC:",  roc)



'''
Created on Mar 5, 2018

@author: khiem
'''
from pyspark.ml.classification import MultilayerPerceptronClassifier
from pyspark.ml.evaluation import MulticlassClassificationEvaluator
from pyspark.sql import SparkSession

spark = SparkSession\
        .builder\
        .appName("NN")\
        .getOrCreate()
# Load training data
data = spark.read.format("libsvm")\
    .load("/usr/local/spark/data/mllib/sample_multiclass_classification_data.txt")

# Split the data into train and test
splits = data.randomSplit([0.6, 0.4], 1234)
train = splits[0]
test = splits[1]

# specify layers for the neural network:
# input layer of size 4 (features), two intermediate of size 5 and 4
# and output of size 3 (classes)
layers = [4, 5, 4, 3]

# create the trainer and set its parameters
trainer = MultilayerPerceptronClassifier(maxIter=100, layers=layers, blockSize=128, seed=1234)

# train the model
model = trainer.fit(train)

# compute accuracy on the test set
result = model.transform(test)
predictionAndLabels = result.select("prediction", "label")
evaluator = MulticlassClassificationEvaluator(metricName="accuracy")
print("Test set accuracy = " + str(evaluator.evaluate(predictionAndLabels)))



'''
Created on Apr 10, 2017

@author: atc
'''
# from pyspark.mllib.clustering import KMeans
from pyspark import SparkConf, SparkContext
import os

# Configure the Spark environment
sparkConf = SparkConf().setAppName("WordCounts").setMaster("local")
sc = SparkContext(conf = sparkConf)

# The WordCounts Spark program
textFile = sc.textFile(os.environ["SPARK_HOME"] + "/README.md")
wordCounts = textFile.flatMap(lambda line: line.split()).map(lambda word: (word, 1)).reduceByKey(lambda a, b: a+b)
print wordCounts.take(5)
# for wc in wordCounts.collect(): print wc

# difference map/flatMap
# print textFile.map(lambda x: x.split(" ")).collect()
# print textFile.flatMap(lambda x: x.split(" ")).collect()


'''
Created on May 17, 2017

@author: atc
'''
# from pyspark.mllib.clustering import KMeans
from pyspark import SparkConf, SparkContext
from pyspark.mllib.util import MLUtils
import os

# Configure the Spark environment
sparkConf = SparkConf().setAppName("WordCounts").setMaster("local")
sc = SparkContext(conf = sparkConf)

from pyspark.mllib.feature import HashingTF
sentence = "hello hello world"
words = sentence.split()  # Split sentence into a list of terms
tf = HashingTF(10000)  # Create vectors of size S = 10,000
tf.transform(words)
print tf
rdd = sc.wholeTextFiles("data").map(lambda (name, text): text.split())
print rdd
tfVectors = tf.transform(rdd)   # Transforms an entire RDD
print tfVectors

from pyspark.mllib.linalg import Vectors
from pyspark.mllib.feature import StandardScaler
vectors = [Vectors.dense([-2.0, 5.0, 1.0]), Vectors.dense([2.0, 0.0, 1.0])]
print vectors
dataset = sc.parallelize(vectors)
scaler = StandardScaler(withMean=True, withStd=True)
model = scaler.fit(dataset)
result = model.transform(dataset)
print result.collect()

# from pyspark.mllib.regression import LabeledPoint 
# from pyspark.mllib.regression import LinearRegressionWithSGD 
# # points = dataset# (create RDD of LabeledPoint) 
# points = MLUtils.loadLibSVMFile(sc, '/usr/local/spark/data/mllib/sample_libsvm_data.txt')
# model = LinearRegressionWithSGD.train(points, iterations=200, intercept=True) 
# print "weights: %s, intercept: %s" % (model.weights, model.intercept) 





from __future__ import print_function
from pyspark.ml.clustering import GaussianMixture
from pyspark.sql import SparkSession
from pyspark import SparkConf, SparkContext

# Configure the Spark environment
sparkConf = SparkConf().setAppName("Classification").setMaster("local")
sc = SparkContext(conf=sparkConf)
"""
A simple example demonstrating Gaussian Mixture Model (GMM).
Run with:
  bin/spark-submit examples/src/main/python/ml/gaussian_mixture_example.py
"""

if __name__ == "__main__":
    spark = SparkSession\
        .builder\
        .appName("GaussianMixtureExample")\
        .getOrCreate()

    # $example on$
    # loads data
    dataset = spark.read.format("libsvm")\
        .load("/usr/local/spark/data/mllib/sample_kmeans_data.txt")

    gmm = GaussianMixture().setK(2).setSeed(538009335)
    model = gmm.fit(dataset)

    print("Gaussians shown as a DataFrame: ")
    model.gaussiansDF.show(truncate=False)
    # $example off$

    spark.stop()



from pyspark import SparkContext
from pyspark.mllib.regression import LabeledPoint
from pyspark.mllib.classification import LogisticRegressionWithSGD
from pyspark.mllib.feature import HashingTF

if __name__ == "__main__":
    sc = SparkContext(appName="MailPrediction")

    # Load 2 types of emails from text files: spam and ham (non-spam).
    # Each line has text from one email.
    spam = sc.textFile("/home/atc/workspace/SparkNew/data/spam.txt")
    ham = sc.textFile("/home/atc/workspace/SparkNew/data/ham.txt")

    # Create a HashingTF instance to map email text to vectors of 100 features.
    tf = HashingTF(numFeatures = 100)
    # Each email is split into words, and each word is mapped to one feature.
    spamFeatures = spam.map(lambda email: tf.transform(email.split(" ")))
    hamFeatures = ham.map(lambda email: tf.transform(email.split(" ")))

    # Create LabeledPoint datasets for positive (spam) and negative (ham) examples.
    positiveExamples = spamFeatures.map(lambda features: LabeledPoint(1, features))
    negativeExamples = hamFeatures.map(lambda features: LabeledPoint(0, features))
    training_data = positiveExamples.union(negativeExamples)
    training_data.cache() # Cache data since Logistic Regression is an iterative algorithm.

    # Run Logistic Regression using the SGD optimizer.
    # regParam is model regularization, which can make models more robust.
    model = LogisticRegressionWithSGD.train(training_data)

    # Test on a positive example (spam) and a negative one (ham).
    # First apply the same HashingTF feature transformation used on the training data.
    posTestExample = tf.transform("O M G GET cheap stuff by sending money to ...".split(" "))
    negTestExample = tf.transform("Hi Dad, I started studying Spark the other ...".split(" "))

    # Now use the learned model to predict spam/ham for new emails.
    a = model.predict(posTestExample)
    print "Prediction for positive test example: %g" % model.predict(posTestExample)
    print "Prediction for negative test example: %g" % model.predict(negTestExample)
    
    sc.stop()



'''
Created on Nov 21, 2017

@author: khiem
'''
from pyspark import SparkConf, SparkContext
from pyspark.sql import SQLContext
from pyspark.sql import Row, Column
from pyspark.sql.functions import rand, randn, struct

# Configure the Spark environment
sparkConf = SparkConf().setAppName("Classification").setMaster("local")
sc = SparkContext(conf=sparkConf)

sqlContext = SQLContext(sc)

# data frame
lo = [('Ankit',25), ('Jalfaizy',22), ('saurabh',20), ('Bala',26)]
rdd = sc.parallelize(lo)
people = rdd.map(lambda x: Row(name=x[0], age=int(x[1])))
schemaPeople = sqlContext.createDataFrame(people)


# query
sqlContext.registerDataFrameAsTable(schemaPeople, 'train_table')
# temporary , exist with session
schemaPeople.createOrReplaceTempView("people")
# Register the DataFrame as a global temporary view
schemaPeople.createGlobalTempView("people")
# sql return dataFrame
sqlContext.sql('select name from train_table')  # .show(5)

# Random Data Generation
df = sqlContext.range(0, 4)
df.select(df.id, rand(2).alias("uniform"), randn(4).alias("standard normal"))\
#     .show()
print df  # note: no include uniform col ?

# Summary and Descriptive Statistics
df.describe(["id"]).show()
from pyspark.sql.functions import mean, min, max
df.select([mean("id"), min("id")])  # .show()

# Sample covariance and correlation
df = sqlContext.range(0, 5)\
    .withColumn("rand1", rand(6))\
    .withColumn("rand2", rand(8))
cor = df.stat.corr("rand1", "rand2"); print cor
cov = df.stat.cov('id', 'id')

# Cross Tabulation (Contingency Table)
names = ["Alice", "Bob", "Mike"]
items = ["milk", "bread", "butter", "apples", "oranges"]
df = sqlContext.createDataFrame(
    [(names[i % 3], items[i % 5]) for i in range(100)], ["name", "item"])

df.crosstab("name", "item").show()

# Frequent Items, support 40%
df = sqlContext.createDataFrame(
    [(1, 2, 3) if i % 2 == 0 else (i, 2 * i, i % 4)
     for i in range(10)], ["a", "b", "c"])
# df.show(12)
freq = df.stat.freqItems(["a", "b", "c"], 0.4)
# freq.show()
freq = df.withColumn('ab', struct('a', 'b')).stat.freqItems(['ab'], 0.4)
freq.show()

# Inferring the Schema Using Reflection
# Load a text file and convert each line to a Row.
lines = sc.textFile("/usr/local/spark/examples/src/main/resources/people.txt")
parts = lines.map(lambda l: l.split(","))
people = parts.map(lambda p: Row(name=p[0], age=int(p[1])))
schemaPeople = sc.createDataFrame(people)

# The schema is encoded in a string.
# schemaString = "name age"
# fields = [StructField(field_name, StringType(), True) for field_name in schemaString.split()]
# schema = StructType(fields)





'''
Created on May 15, 2017

@author: atc
'''
from pyspark.mllib.tree import DecisionTree, DecisionTreeModel
from pyspark.mllib.util import MLUtils
from pyspark import SparkConf, SparkContext

# Configure the Spark environment
sparkConf = SparkConf().setAppName("Classification").setMaster("local")
sc = SparkContext(conf = sparkConf)

# Load and parse the data file into an RDD of LabeledPoint.
data = MLUtils.loadLibSVMFile(
    sc, '/usr/local/spark/data/mllib/sample_libsvm_data.txt')
# Split the data into training and test sets (30% held out for testing)
(trainingData, testData) = data.randomSplit([0.7, 0.3])

# Train a DecisionTree model.
#  Empty categoricalFeaturesInfo indicates all features are continuous.
model = DecisionTree.trainClassifier(
    trainingData, numClasses=2, categoricalFeaturesInfo={},
    impurity='gini', maxDepth=5, maxBins=32
    )
# Evaluate model on test instances and compute test error
predictions = model.predict(testData.map(lambda x: x.features))
labelsAndPredictions = testData.map(lambda lp: lp.label).zip(predictions)
print labelsAndPredictions.collect()
testErr = labelsAndPredictions.filter(
    lambda (v, p): v != p).count() / float(testData.count()
    )
print('Test Error = ' + str(testErr))
print('Learned classification tree model:')
print(model.toDebugString())

# Save and load model
model.save(sc, "target/tmp/myDecisionTreeClassificationModel")
sameModel = DecisionTreeModel.load(sc, "target/tmp/myDecisionTreeClassificationModel")



'''
Created on Mar 9, 2018

@author: khiem
'''
from pyspark.mllib.classification import SVMWithSGD, SVMModel
from pyspark.mllib.regression import LabeledPoint
from pyspark import SparkConf, SparkContext

# Configure the Spark environment
sparkConf = SparkConf().setAppName("Classification").setMaster("local")
sc = SparkContext(conf=sparkConf)

# Load and parse the data
def parsePoint(line):
    values = [float(x) for x in line.split(' ')]
    return LabeledPoint(values[0], values[1:])


data = sc.textFile("/usr/local/spark/data/mllib/sample_svm_data.txt")
parsedData = data.map(parsePoint)

# Build the model
model = SVMWithSGD.train(parsedData, iterations=100)

# Evaluating the model on training data
labelsAndPreds = parsedData.map(lambda p: (p.label, model.predict(p.features)))
trainErr = labelsAndPreds.filter(
    lambda lp: lp[0] != lp[1]).count() / float(parsedData.count())
print("Training Error = " + str(trainErr))

# Save and load model
# model.save(sc, "target/tmp/pythonSVMWithSGDModel")
# sameModel = SVMModel.load(sc, "target/tmp/pythonSVMWithSGDModel")



'''
Created on Mar 9, 2018

@author: khiem
@python3

'''
from setuptools import setup, find_packages

setup(name='xinsonha',
      version='0.1',
      url='https://github.com/the-gigi/pathology',
      license='MIT',
      author='Gigi Sayfan',
      author_email='the.gigi@gmail.com',
      description='Add static script_dir() method to Path',
      packages=find_packages(exclude=['tests']),
#       long_description=open('README.md').read(),
      zip_safe=False)



import pathlib
import inspect 

# script_dir = pathlib.Path(__file__).parent.resolve()
# print (script_dir)
# print(open(str(script_dir/'markdown.ipynb').read())


class Path(type(pathlib.Path())):
    @staticmethod
    def script_dir():
        file_name = inspect.stack()[1].filename
        p = pathlib.Path(file_name)
        return p.parent.resolve()

# print (Path.script_dir())



from __future__ import print_function
from pyspark.ml.clustering import KMeans
from pyspark.sql import SparkSession
 
"""
An example demonstrating k-means clustering.
Run with:
  bin/spark-submit examples/src/main/python/ml/kmeans_example.py
"""
 
if __name__ == "__main__":
    spark = SparkSession\
        .builder\
        .appName("KMeansExample")\
        .getOrCreate()
 
    # Loads data.
    dataset = spark.read.format("libsvm")\
        .load("/usr/local/spark/data/mllib/sample_kmeans_data.txt")
 
    # Trains a k-means model.
    kmeans = KMeans().setK(2).setSeed(1)
    model = kmeans.fit(dataset)
 
    # Evaluate clustering by computing Within Set Sum of Squared Errors.
    wssse = model.computeCost(dataset)
    print("Within Set Sum of Squared Errors = " + str(wssse))
 
    # Shows the result.
    centers = model.clusterCenters()
    print("Cluster Centers: ")
    for center in centers:
        print(center)
    # $example off$
 
    spark.stop()


# Hierachy cluster
# from pyspark.ml.clustering import BisectingKMeans
# 
# spark = SparkSession\
#         .builder\
#         .appName("KMeansExample")\
#         .getOrCreate()
# # Loads data.
# dataset = spark.read.format("libsvm").load("/usr/local/spark/data/mllib/sample_kmeans_data.txt")
# 
# # Trains a bisecting k-means model.
# bkm = BisectingKMeans().setK(2).setSeed(1)
# model = bkm.fit(dataset)
# 
# # Evaluate clustering.
# cost = model.computeCost(dataset)
# print("Within Set Sum of Squared Errors = " + str(cost))
# 
# # Shows the result.
# print("Cluster Centers: ")
# centers = model.clusterCenters()
# for center in centers:
#     print(center)


# kmean low level
# from numpy import array
# from math import sqrt
# from pyspark import SparkConf, SparkContext
# from pyspark.mllib.clustering import KMeans, KMeansModel
#   
# # Configure the Spark environment
# sparkConf = SparkConf().setAppName("Classification").setMaster("local")
# sc = SparkContext(conf=sparkConf)
#   
# # Load and parse the data
# data = sc.textFile("/usr/local/spark/data/mllib/sample_kmeans_data.txt")
# parsedData = data.map(lambda line: array([float(x) for x in line.split(' ')]))
#   
# # Build the model (cluster the data)
# clusters = KMeans.train(
#     parsedData, 2, maxIterations=10, initializationMode="random")
#   
# # Evaluate clustering by computing Within Set Sum of Squared Errors
# def error(point):
#     center = clusters.centers[clusters.predict(point)]
#     return sqrt(sum([x**2 for x in (point - center)]))
#   
# WSSSE = parsedData.map(lambda point: error(point)).reduce(lambda x, y: x + y)
# print("Within Set Sum of Squared Error = " + str(WSSSE))
#   
# # Save and load model
# clusters.save(sc, "target/org/apache/spark/PythonKMeansExample/KMeansModel")
# sameModel = KMeansModel.load(sc, "target/org/apache/spark/PythonKMeansExample/KMeansModel")
